<?php

    use PRAMADILLO\INTEGRATIONS\Elementor;
    use PRAMADILLO\INTEGRATIONS\PaidMembershipsPro;
    use PRAMADILLO\INTEGRATIONS\WooCommerceMemberships;
    use PRAMADILLO\INTEGRATIONS\WooCommerceSubscriptions;

    class Woocommerce_Pay_Per_Post_Admin {

        public $integrations;

        private $allowed_restriction_frequency = ['minute', 'hour', 'day', 'week', 'month', 'year'];

        public function __construct() {
            if (wcppp_freemius()->is__premium_only()) {
                if (wcppp_freemius()->can_use_premium_code()) {
                    if (Woocommerce_Pay_Per_Post_Helper::can_use_woocommerce_memberships()) {
                        $this->integrations['woocommerce-memberships'] = new WooCommerceMemberships();
                    }
                    if (Woocommerce_Pay_Per_Post_Helper::can_use_woocommerce_subscriptions()) {
                        $this->integrations['woocommerce-subscriptions'] = new WooCommerceSubscriptions();
                    }
                    if (Woocommerce_Pay_Per_Post_Helper::can_use_paid_membership_pro()) {
                        $this->integrations['paid-memberships-pro'] = new PaidMembershipsPro();
                    }
                    if (Woocommerce_Pay_Per_Post_Helper::can_use_elementor()) {
                        $this->integrations['elementor'] = new Elementor();
                    }
                }
            }
        }

        public function add_plugin_options() {
            global $wp_version;
            add_menu_page(
                __('Pay For Post with WooCommerce', 'wc_pay_per_post'),
                'Pay For Post',
                'manage_options',
                WC_PPP_SLUG,
                [$this, 'create_options_page'],
                'dashicons-cart',
                99
            );

            add_submenu_page(
                WC_PPP_SLUG,
                'Settings',
                'Settings',
                'manage_options',
                WC_PPP_SLUG.'-settings',
                [$this, 'create_options_page']
            );

            add_submenu_page(
                WC_PPP_SLUG,
                'What\'s New',
                'What\'s New',
                'manage_options',
                WC_PPP_SLUG.'-whats-new',
                [$this, 'create_whatsnew_page']
            );

            add_submenu_page(
                WC_PPP_SLUG,
                'Getting Started with Pay For Post with WooCommerce',
                'Getting Started',
                'manage_options',
                WC_PPP_SLUG.'-getting-started',
                [$this, 'create_getting_started_page']
            );

            if (wcppp_freemius()->is__premium_only()) {
                if (wcppp_freemius()->can_use_premium_code()) {
                    add_submenu_page(
                        WC_PPP_SLUG,
                        'Pay For Post with WooCommerce Protected Content',
                        'Protected Content',
                        'manage_options',
                        WC_PPP_SLUG.'-protected-content',
                        [$this, 'create_protected_content_page']
                    );
                }
            }

            add_submenu_page(
                WC_PPP_SLUG,
                'Pay For Post with WooCommerce Documentation',
                'Documentation',
                'manage_options',
                WC_PPP_SLUG.'-help',
                [$this, 'create_help_page']
            );

            if (version_compare($wp_version, '5.2.0', '>=')) {

                add_submenu_page(
                    WC_PPP_SLUG,
                    'Debug',
                    'Debug',
                    'manage_options',
                    WC_PPP_SLUG.'-debug',
                    [$this, 'create_debug_page']
                );

            }

            add_submenu_page(
                WC_PPP_SLUG,
                '',
                '<span class="pramadillo-admin-separator-container"><span class="pramadillo-admin-separator-title">Account Info</span><span class="pramadillo-admin-separator"></span></span>',
                'manage_options',
                '#'
            );


            remove_submenu_page(WC_PPP_SLUG, WC_PPP_SLUG);

        }

        public function enqueue_scripts() {

            wp_register_script(WC_PPP_SLUG.'_admin', plugin_dir_url(__FILE__).'js/wc-ppp-admin.js', ['jquery'], WC_PPP_VERSION);
            wp_register_script(WC_PPP_SLUG.'_select2', plugin_dir_url(__FILE__).'js/select2.min.js', [], '4.0.6');

            if (wcppp_freemius()->is__premium_only()) {
                if (wcppp_freemius()->can_use_premium_code()) {
                    wp_register_script(WC_PPP_SLUG.'_dataTables', plugin_dir_url(__FILE__).'js/jquery.dataTables.min.js', ['jquery'], WC_PPP_VERSION);
                    wp_register_script(WC_PPP_SLUG.'_dataTables_user', plugin_dir_url(__FILE__).'js/wc-ppp-user.js', [WC_PPP_SLUG.'_dataTables'], WC_PPP_VERSION);
                    wp_register_script(WC_PPP_SLUG.'_dataTables_content', plugin_dir_url(__FILE__).'js/wc-ppp-purchased-content.js', [WC_PPP_SLUG.'_dataTables'], WC_PPP_VERSION);

                    wp_localize_script(
                        WC_PPP_SLUG.'_dataTables_user', 'wc_ppp', [
                                                          'ajax_url'               => admin_url('admin-ajax.php'),
                                                          'delete_page_view_nonce' => wp_create_nonce('delete_page_view_nonce'),
                                                      ]
                    );

                }
            }

        }

        public function enqueue_styles() {
            wp_register_style(WC_PPP_SLUG.'_admin', plugin_dir_url(__FILE__).'css/wc-ppp-admin.css', [], WC_PPP_VERSION);
            wp_register_style(WC_PPP_SLUG.'_select2', plugin_dir_url(__FILE__).'css/select2.min.css"', [], WC_PPP_VERSION);

            if (wcppp_freemius()->is__premium_only()) {
                if (wcppp_freemius()->can_use_premium_code()) {
                    wp_register_style(WC_PPP_SLUG.'_dataTables', plugin_dir_url(__FILE__).'css/jquery.dataTables.min.css"', [], WC_PPP_VERSION, false);
                }
            }

        }

        public function admin_menu_separator_styles() {
            ?>
            <style>
                .pramadillo-admin-separator-container {
                    display: flex;
                    height: 12px;
                    align-items: center;
                    margin: 0 -10px 0 0;
                }

                .pramadillo-admin-separator-container .pramadillo-admin-separator-title {
                    font-size: .68em;
                    text-transform: uppercase;
                    font-weight: 700;
                    margin-right: 10px;
                    color: hsla(0, 0%, 100%, .25);
                }

                .pramadillo-admin-separator-container .pramadillo-admin-separator {
                    display: block;
                    flex: 1;
                    padding: 0;
                    height: 1px;
                    line-height: 1px;
                    background: hsla(0, 0%, 100%, .125);
                }
            </style>
            <?php
        }

        public function ajax_post_types(): array {
            $post_types = [];
            foreach (get_post_types(['public' => true]) as $post_type) {
                $post_types[] = $post_type;
            }

            return $post_types;
        }

        public function options_init() {
            register_setting(WC_PPP_SLUG.'_settings', WC_PPP_SLUG.'_restricted_content_default');
            register_setting(WC_PPP_SLUG.'_settings', WC_PPP_SLUG.'_custom_post_types');
            register_setting(WC_PPP_SLUG.'_settings', WC_PPP_SLUG.'_only_show_virtual_products');
            register_setting(WC_PPP_SLUG.'_settings', WC_PPP_SLUG.'_turn_off_comments_when_protected');
            register_setting(WC_PPP_SLUG.'_settings', WC_PPP_SLUG.'_allow_admins_access_to_protected_posts');
            register_setting(WC_PPP_SLUG.'_settings', WC_PPP_SLUG.'_enable_debugging');
            register_setting(WC_PPP_SLUG.'_settings', WC_PPP_SLUG.'_delete_settings');

            if (wcppp_freemius()->is__premium_only()) {
                if (wcppp_freemius()->can_use_premium_code()) {
                    register_setting(WC_PPP_SLUG.'_settings', WC_PPP_SLUG.'_turn_off_comments_public');
                    register_setting(WC_PPP_SLUG.'_settings', WC_PPP_SLUG.'_display_on_my_account');
                    register_setting(WC_PPP_SLUG.'_settings', WC_PPP_SLUG.'_enable_polylang');
                    register_setting(WC_PPP_SLUG.'_settings', WC_PPP_SLUG.'_show_locks');
                }
            }
        }

        /** @noinspection PhpIncludeInspection */
        public function create_options_page() {
            global $wcppp_freemius;

            $restricted_content_default = get_option(WC_PPP_SLUG.'_restricted_content_default', _x("<h1>Oops, Restricted Content</h1>\n<p>We are sorry but this post is restricted to folks that have purchased this page.</p>\n\n[products ids='{{product_id}}' columns='1']", 'Default restricted content', 'wc_pay_per_post'));
            $custom_post_types          = get_option(WC_PPP_SLUG.'_custom_post_types', []);
            $custom_post_types          = empty($custom_post_types) ? [] : $custom_post_types;

            if ( ! is_array($custom_post_types)) {
                $custom_post_types = explode(',', $custom_post_types);
            }

            $turn_off_comments_when_protected = get_option(WC_PPP_SLUG.'_turn_off_comments_when_protected', true);
            if (wcppp_freemius()->is__premium_only()) {
                if (wcppp_freemius()->can_use_premium_code()) {
                    $turn_off_comments_public    = get_option(WC_PPP_SLUG.'_turn_off_comments_public', false);
                    $display_on_my_account       = get_option(WC_PPP_SLUG.'_display_on_my_account', false);
                    $enable_polylang             = get_option(WC_PPP_SLUG.'_enable_polylang', false);
                    $memberships_active          = (isset($this->integrations['woocommerce-memberships']));
                    $subscriptions_active        = (isset($this->integrations['woocommerce-subscriptions']));
                    $paid_memberships_pro_active = (isset($this->integrations['paid-memberships-pro']));
                    $elementor_active            = (isset($this->integrations['elementor']));
                    $show_locks                  = (boolean) get_option(WC_PPP_SLUG.'_show_locks', false);
                }
            }
            $allow_admins_access_to_protected_posts = (boolean) get_option(WC_PPP_SLUG.'_allow_admins_access_to_protected_posts', false);
            $enable_debugging                       = (boolean) get_option(WC_PPP_SLUG.'_enable_debugging', false);
            $delete_settings                        = (boolean) get_option(WC_PPP_SLUG.'_delete_settings', false);
            $available_post_types                   = $this->ajax_post_types();
            $only_show_virtual_products             = (boolean) get_option(WC_PPP_SLUG.'_only_show_virtual_products', false);

            if ( ! current_user_can('manage_options')) {
                wp_die(esc_html('You do not have sufficient permissions to access this page.'));
            }
            wp_enqueue_style(WC_PPP_SLUG.'_admin');
            wp_enqueue_style(WC_PPP_SLUG.'_select2');
            wp_enqueue_script(WC_PPP_SLUG.'_select2');
            wp_enqueue_script(WC_PPP_SLUG.'_admin');


            //Delete Log Functionality
            if (isset($_GET['delete_log_nonce']) && wp_verify_nonce($_GET['delete_log_nonce'], 'delete_log')) {
                $log = new Woocommerce_Pay_Per_Post_Logger();
                $log->delete_log_file();
                echo '<script>alert("Log File Deleted");</script>';
            }


            require_once plugin_dir_path(__FILE__).'partials/settings.php';
        }

        public function maybe_flush_permalinks__premium_only($old_value, $value, $option) {
            if ($value == 1) {
                flush_rewrite_rules();
            }
        }

        public function create_help_page() {
            wp_enqueue_style(WC_PPP_SLUG.'_admin');
            wp_enqueue_style(WC_PPP_SLUG.'_select2');
            wp_enqueue_script(WC_PPP_SLUG.'_select2');
            wp_enqueue_script(WC_PPP_SLUG.'_admin');
            /** @noinspection PhpIncludeInspection */
            require_once plugin_dir_path(__FILE__).'partials/help.php';
        }

        public function create_protected_content_page() {
            if (is_admin()) {
                wp_enqueue_style(WC_PPP_SLUG.'_admin');

                wp_enqueue_script(WC_PPP_SLUG.'_dataTables');
                wp_enqueue_script(WC_PPP_SLUG.'_dataTables_content');
                wp_enqueue_style(WC_PPP_SLUG.'_dataTables');

                $posts = Woocommerce_Pay_Per_Post_Helper::get_protected_posts();
                $data  = [];
                Woocommerce_Pay_Per_Post_Helper::logger('create_protected_content_page() about to loop through posts.');

                foreach ($posts as $post) {
                    Woocommerce_Pay_Per_Post_Helper::logger('Checking Post - ' . $post->ID . ' for products.');

                    $users           = [];
                    $products        = (array) get_post_meta($post->ID, WC_PPP_SLUG.'_product_ids', true);
                    Woocommerce_Pay_Per_Post_Helper::logger('Products - ' . print_r($products, true));

                    $data[$post->ID] = [
                        'title'    => $post->post_title,
                        'products' => $products,
                    ];
                    foreach ($products as $product) {
                        if ($product != '' || ! is_null($product) || ! empty($product)) {
                            $users[] = $this->get_users_by_product_id($product);
                        }
                    }

                    $data[$post->ID]['users'] = call_user_func_array('array_merge', $users);

                }

                /** @noinspection PhpIncludeInspection */
                require_once plugin_dir_path(__FILE__).'partials/protected-content.php';
            }
        }

        /** @noinspection PhpUnnecessaryCurlyVarSyntaxInspection
         * @noinspection PhpUnnecessaryCurlyVarSyntaxInspection
         */
        protected function get_users_by_product_id($product_id): array {
            global $wpdb;

            $statuses = array_map('esc_sql', wc_get_is_paid_statuses());

            /** @noinspection PhpUnnecessaryCurlyVarSyntaxInspection */
            /** @noinspection PhpUnnecessaryCurlyVarSyntaxInspection */
            return $wpdb->get_col("
           SELECT DISTINCT pm.meta_value FROM {$wpdb->posts} AS p
           INNER JOIN {$wpdb->postmeta} AS pm ON p.ID = pm.post_id
           INNER JOIN {$wpdb->prefix}woocommerce_order_items AS i ON p.ID = i.order_id
           INNER JOIN {$wpdb->prefix}woocommerce_order_itemmeta AS im ON i.order_item_id = im.order_item_id
           WHERE p.post_status IN ( 'wc-".implode("','wc-", $statuses)."' )
           AND pm.meta_key IN ( '_billing_email' )
           AND im.meta_key IN ( '_product_id', '_variation_id' )
           AND im.meta_value = $product_id
        ");
        }

        public function create_debug_page() {
            wp_enqueue_style(WC_PPP_SLUG.'_admin');
            $enable_debugging = (boolean) get_option(WC_PPP_SLUG.'_enable_debugging', false);
            $transients = Woocommerce_Pay_Per_Post_Debug::get_transients();
            if (isset($_POST['wc_ppp_create_table_nonce']) && wp_verify_nonce($_POST['wc_ppp_create_table_nonce'], 'wc_ppp_create_table')) {
                /** @noinspection PhpIncludeInspection */
                require_once WC_PPP_PATH.'/includes/class-woocommerce-pay-per-post-activator.php';
                Woocommerce_Pay_Per_Post_Activator::create_table__premium_only();
            }

	        if (isset($_POST['wc_ppp_delete_transients_nonce']) && wp_verify_nonce($_POST['wc_ppp_delete_transients_nonce'], 'wc_ppp_delete_transients')) {
		       Woocommerce_Pay_Per_Post_Debug::delete_transients();
	        }


            /** @noinspection PhpIncludeInspection */
            require_once plugin_dir_path(__FILE__).'partials/debug.php';
        }

        public function create_getting_started_page() {
            wp_enqueue_style(WC_PPP_SLUG.'_admin');
            /** @noinspection PhpIncludeInspection */
            require_once plugin_dir_path(__FILE__).'partials/getting-started.php';
        }

        public function create_whatsnew_page() {
            wp_enqueue_style(WC_PPP_SLUG.'_admin');

            $needs_upgrade     = get_option(WC_PPP_SLUG.'_needs_upgrade', 'true');
            $custom_post_types = get_option(WC_PPP_SLUG.'_custom_post_types', []);
            $custom_post_types = empty($custom_post_types) ? [] : $custom_post_types;

            if ( ! is_array($custom_post_types)) {
                $custom_post_types = explode(',', $custom_post_types);
            }

            $old_products = new WP_Query(
                [
                    'post_type' => $custom_post_types,
                    'meta_key'  => 'woocommerce_ppp_product_id',
                    'nopaging'  => true,
                ]
            );

            if (isset($_POST['wc_ppp_upgrade_nonce']) && wp_verify_nonce($_POST['wc_ppp_upgrade_nonce'], 'wc_ppp_upgrade')) {
                $this->upgrade_database($old_products);
            }

            $readme          = file_get_contents(WC_PPP_PATH.'README.txt');
            $changelog       = explode('== Changelog ==', $readme);
            $changelog       = explode('== ', $changelog[1]);
            $full_change_log = $changelog[0];
            $last_change     = explode('=', $full_change_log);
            $last_change     = $last_change[2];

            /** @noinspection PhpIncludeInspection */
            require_once plugin_dir_path(__FILE__).'partials/whats-new.php';
        }

        public function meta_box() {

            $allowed_roles = apply_filters('wc_pay_per_post_allowed_roles_for_meta_box', []);

            if (count($allowed_roles) == 0) {
                $allow_meta = true;
            } else {
                $allow_meta = false;
                $user       = wp_get_current_user();
                if ( ! is_null($user)) {
                    $user_roles = ( array ) $user->roles;
                    foreach ($user_roles as $role) {
                        if (array_key_exists($role, $allowed_roles)) {
                            $allow_meta = true;
                        }
                    }

                }

            }

            if ($allow_meta) {

                $post_types = $this->get_post_types();

                foreach ($post_types as $post_type) {
                    add_meta_box(WC_PPP_SLUG.'_meta_box', __('Pay For Post with WooCommerce', 'wc_pay_per_post'), [
                        $this,
                        'output_meta_box',
                    ], $post_type, 'normal', 'high',
                                 [
                                     '__block_editor_compatible_meta_box' => true,
                                 ]);
                }
            }

        }

        public function get_post_types(): array {
            return Woocommerce_Pay_Per_Post_Helper::get_post_types();
        }

        public function output_meta_box() {

            ob_start();
            global $post;
            $id                                      = $post->ID;
            $selected                                = get_post_meta($id, WC_PPP_SLUG.'_product_ids', true);
            $restricted_content_override             = get_post_meta($id, WC_PPP_SLUG.'_restricted_content_override', true);
            $delay_restriction_enable                = get_post_meta($id, WC_PPP_SLUG.'_delay_restriction_enable', true);
            $delay_restriction                       = get_post_meta($id, WC_PPP_SLUG.'_delay_restriction', true);
            $delay_restriction_frequency             = get_post_meta($id, WC_PPP_SLUG.'_delay_restriction_frequency', true);
            $page_view_restriction_enable            = get_post_meta($id, WC_PPP_SLUG.'_page_view_restriction_enable', true);
            $page_view_restriction                   = get_post_meta($id, WC_PPP_SLUG.'_page_view_restriction', true);
            $page_view_restriction_frequency         = get_post_meta($id, WC_PPP_SLUG.'_page_view_restriction_frequency', true);
            $page_view_restriction_enable_time_frame = get_post_meta($id, WC_PPP_SLUG.'_page_view_restriction_enable_time_frame', true);
            $page_view_restriction_time_frame        = get_post_meta($id, WC_PPP_SLUG.'_page_view_restriction_time_frame', true);
            $expire_restriction_enable               = get_post_meta($id, WC_PPP_SLUG.'_expire_restriction_enable', true);
            $expire_restriction                      = get_post_meta($id, WC_PPP_SLUG.'_expire_restriction', true);
            $expire_restriction_frequency            = get_post_meta($id, WC_PPP_SLUG.'_expire_restriction_frequency', true);
            $show_warnings                           = get_post_meta($id, WC_PPP_SLUG.'_show_warnings', true);

            $drop_down = $this->generate_products_dropdown($selected);

            wp_enqueue_style(WC_PPP_SLUG.'_admin');
            wp_enqueue_style(WC_PPP_SLUG.'_select2');
            wp_enqueue_script(WC_PPP_SLUG.'_select2');
            wp_enqueue_script(WC_PPP_SLUG.'_admin');
            /** @noinspection PhpIncludeInspection */
            require plugin_dir_path(dirname(__FILE__)).'admin/partials/meta-box-base.php';
            echo ob_get_clean();

        }

        public function get_all_product_args($args) {
            return $args;
        }

        public function get_virtual_product_args($args) {
            return $args;
        }

        public function save_meta_box($post_id) {

            // Stop the script when doing autosave.
            if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
                return;
            }

            // Verify the nonce. If insn't there, stop the script.
            if ( ! isset($_POST[WC_PPP_SLUG.'_nonce']) || ! wp_verify_nonce($_POST[WC_PPP_SLUG.'_nonce'], WC_PPP_SLUG.'_nonce')) {
                return;
            }
            // Stop the script if the user does not have edit permissions.
            if ( ! current_user_can('edit_posts')) {
                return;
            }

            // Boolean Values.
            $_delay_restriction_enable                = 0;
            $_page_view_restriction_enable            = 0;
            $_page_view_restriction_enable_time_frame = 0;
            $_expire_restriction_enable               = 0;
            $_show_warnings                           = 0;

            // Save the product_id's associated with page/post.
            $product_ids = ($_POST[WC_PPP_SLUG.'_product_ids'] ?? '');
            $product_ids = $this->sanitize_product_ids($product_ids);
            update_post_meta($post_id, WC_PPP_SLUG.'_product_ids', $product_ids);

            if (wcppp_freemius()->is__premium_only()) {
                if (wcppp_freemius()->can_use_premium_code()) {
                    // Save the custom override if any.

                    if (isset($_POST[WC_PPP_SLUG.'_restricted_content_override'])) {
                        update_post_meta($post_id, WC_PPP_SLUG.'_restricted_content_override', wp_filter_post_kses($_POST[WC_PPP_SLUG.'_restricted_content_override']));
                    }

                    // Enable Delay Restriction.
                    if (isset($_POST[WC_PPP_SLUG.'_delay_restriction_enable'])) {
                        $_delay_restriction_enable = 1;
                    }
                    update_post_meta($post_id, WC_PPP_SLUG.'_delay_restriction_enable', (bool) $_delay_restriction_enable);

                    // Length Delay Restriction.
                    if (isset($_POST[WC_PPP_SLUG.'_delay_restriction'])) {
                        update_post_meta($post_id, WC_PPP_SLUG.'_delay_restriction', (int) $_POST[WC_PPP_SLUG.'_delay_restriction']);
                    }

                    // Delay Restriction Frequency.
                    if (isset($_POST[WC_PPP_SLUG.'_delay_restriction_frequency']) && in_array($_POST[WC_PPP_SLUG.'_delay_restriction_frequency'], $this->allowed_restriction_frequency, true)) {
                        update_post_meta($post_id, WC_PPP_SLUG.'_delay_restriction_frequency', sanitize_text_field($_POST[WC_PPP_SLUG.'_delay_restriction_frequency']));
                    }

                    // Enable page_view Restriction.
                    if (isset($_POST[WC_PPP_SLUG.'_page_view_restriction_enable'])) {
                        $_page_view_restriction_enable = 1;
                    }
                    update_post_meta($post_id, WC_PPP_SLUG.'_page_view_restriction_enable', (bool) $_page_view_restriction_enable);

                    // _page_view_restriction
                    if (isset($_POST[WC_PPP_SLUG.'_page_view_restriction'])) {
                        update_post_meta($post_id, WC_PPP_SLUG.'_page_view_restriction', (int) $_POST[WC_PPP_SLUG.'_page_view_restriction']);
                    }

                    // _page_view_restriction_enable_time_frame
                    if (isset($_POST[WC_PPP_SLUG.'_page_view_restriction_enable_time_frame'])) {
                        $_page_view_restriction_enable_time_frame = 1;
                    }
                    update_post_meta($post_id, WC_PPP_SLUG.'_page_view_restriction_enable_time_frame', (bool) $_page_view_restriction_enable_time_frame);

                    // Length page_view Restriction.
                    if (isset($_POST[WC_PPP_SLUG.'_page_view_restriction_time_frame'])) {
                        update_post_meta($post_id, WC_PPP_SLUG.'_page_view_restriction_time_frame', (int) $_POST[WC_PPP_SLUG.'_page_view_restriction_time_frame']);
                    }

                    // page_view Restriction Frequency.
                    if (isset($_POST[WC_PPP_SLUG.'_page_view_restriction_frequency']) && in_array($_POST[WC_PPP_SLUG.'_page_view_restriction_frequency'], $this->allowed_restriction_frequency, true)) {
                        update_post_meta($post_id, WC_PPP_SLUG.'_page_view_restriction_frequency', sanitize_text_field($_POST[WC_PPP_SLUG.'_page_view_restriction_frequency']));
                    }

                    // Enable expire Restriction.
                    if (isset($_POST[WC_PPP_SLUG.'_expire_restriction_enable'])) {
                        $_expire_restriction_enable = 1;
                    }
                    update_post_meta($post_id, WC_PPP_SLUG.'_expire_restriction_enable', (bool) $_expire_restriction_enable);

                    // Length expire Restriction.
                    if (isset($_POST[WC_PPP_SLUG.'_expire_restriction'])) {
                        update_post_meta($post_id, WC_PPP_SLUG.'_expire_restriction', (int) $_POST[WC_PPP_SLUG.'_expire_restriction']);
                    }

                    // expire Restriction Frequency.
                    if (isset($_POST[WC_PPP_SLUG.'_expire_restriction_frequency']) && in_array($_POST[WC_PPP_SLUG.'_expire_restriction_frequency'], $this->allowed_restriction_frequency, true)) {
                        update_post_meta($post_id, WC_PPP_SLUG.'_expire_restriction_frequency', sanitize_text_field($_POST[WC_PPP_SLUG.'_expire_restriction_frequency']));
                    }

                    // Save Options.

                    if (isset($_POST[WC_PPP_SLUG.'_show_warnings'])) {
                        $_show_warnings = 1;
                    }
                    update_post_meta($post_id, WC_PPP_SLUG.'_show_warnings', (bool) $_show_warnings);

                }
            }
        }

        public function manage_custom_column($column, $post_id) {
            if ($column === WC_PPP_SLUG.'_protected') {

                $protected = Woocommerce_Pay_Per_Post_Helper::is_protected($post_id);
                if ($protected) {
                    echo Woocommerce_Pay_Per_Post_Helper::protection_display_icon($protected);
                }
            }
        }

        public function manage_columns($columns) {
            $columns[WC_PPP_SLUG.'_protected'] = 'Pay For Post';

            return $columns;
        }

        public function sortable_columns($columns) {
            $columns[WC_PPP_SLUG.'_protected'] = WC_PPP_SLUG.'_protected';

            return $columns;
        }

        public function plugin_settings_link($links) {
            $url     = admin_url('options-general.php?page='.WC_PPP_SLUG);
            $_link   = '<a href="'.$url.'">'.__('Settings', 'wc_pay_per_post').'</a>';
            $links[] = $_link;

            return $links;
        }

        public function prefix_plugin_update_message($data, $response) {
            if (isset($data['upgrade_notice'])) {
                printf(
                    '<div class="update-message">%s</div>',
                    esc_html(wpautop($data['upgrade_notice']))
                );
            }
        }

        public function extra_user_profile_fields__premium_only($user) {

            $user          = wp_get_current_user();
            $allowed_roles = apply_filters('wc_pay_per_post_allowed_roles_for_user_profile_edit', ['administrator']);

            if (array_intersect($allowed_roles, $user->roles)) {

                ob_start();

                wp_enqueue_script(WC_PPP_SLUG.'_dataTables');
                wp_enqueue_script(WC_PPP_SLUG.'_dataTables_user');
                wp_enqueue_style(WC_PPP_SLUG.'_dataTables');

                $user_table = new Woocommerce_Pay_Per_Post_Users_Table();
                $page_views = $user_table->get_user_page_views__premium_only();

                /** @noinspection PhpIncludeInspection */
                require plugin_dir_path(dirname(__FILE__)).'admin/partials/users-table__premium_only.php';
                echo ob_get_clean();
            }
        }

        public function ajax_delete_page_view__premium_only() {
            if (check_ajax_referer('delete_page_view_nonce', 'security')) {
                global $wpdb;
                $page_id = (int) trim($_REQUEST['data']);

                return $wpdb->delete(
                    $wpdb->prefix.'woocommerce_pay_per_post_pageviews',
                    [
                        'id' => $page_id,
                    ],
                    [
                        'id' => '%d',
                    ]
                );
            }

            wp_die();

        }

        protected function upgrade_database($products) {

            foreach ($products->posts as $post) {
                // Get old meta key for product id's associated with posts.
                $post_meta = get_post_meta($post->ID, 'woocommerce_ppp_product_id', true);

                if ('' !== $post_meta) {  // Added in to account for fields that were there but with no products associated with them.
                    $old_ppp_ids = explode(',', $post_meta);
                    update_post_meta($post->ID, WC_PPP_SLUG.'_product_ids', $old_ppp_ids);
                }
            }

            update_option('wc_pay_per_post_needs_upgrade', 'false', false);
            update_option('wc_pay_per_post_db_version', WC_PPP_PLUGIN_VERSION, false);
            $url = admin_url('admin.php?page='.WC_PPP_SLUG.'-whats-new&upgrade_complete=true');
            wp_safe_redirect($url);
        }

        protected function get_post_products_custom_field($value, $id = null) {
            $custom_field = get_post_meta($id, $value, true);
            if ( ! empty($custom_field)) {
                return is_array($custom_field) ? stripslashes_deep($custom_field) : stripslashes(wp_kses_decode_entities($custom_field));
            } else {
                return false;
            }
        }

        protected function generate_products_dropdown($selected = []): string {

            $products = Woocommerce_Pay_Per_Post_Helper::get_products();

            $drop_down = '<select id="'.WC_PPP_SLUG.'_product_ids" name="'.WC_PPP_SLUG.'_product_ids[]" style="width: 100%" multiple="multiple">';

            if (wcppp_freemius()->is__premium_only()) {
                if (wcppp_freemius()->can_use_premium_code()) {

                    if (isset($this->integrations['woocommerce-memberships'])) {
                        $drop_down .= $this->integrations['woocommerce-memberships']->generate_membership_levels_dropdown($selected);
                    }

                    if (isset($this->integrations['woocommerce-subscriptions'])) {
                        $all_products              = $products;
                        $non_subscription_products = $this->integrations['woocommerce-subscriptions']->filter_subscription_products($products);
                        $subscription_products     = Woocommerce_Pay_Per_Post_Helper::md_array_diff($all_products, $non_subscription_products);
                        $products                  = $non_subscription_products;

                        $drop_down .= $this->integrations['woocommerce-subscriptions']->generate_subscriptions_dropdown($subscription_products, $selected);
                    }

                    if (isset($this->integrations['paid-memberships-pro'])) {
                        $drop_down .= $this->integrations['paid-memberships-pro']->generate_membership_levels_dropdown($selected);
                    }

                }
            }

            $drop_down .= '<optgroup label="Products">';
            foreach ($products as $product) {
                $drop_down .= '<option value="'.$product['ID'].'"';

                if (in_array((string) $product['ID'], (array) $selected, true)) {
                    $drop_down .= ' selected="selected"';
                }

                $drop_down .= '>'.$product['post_title'].' - [#'.$product['ID'].']</option>';
            }
            $drop_down .= '</optgroup>';


            $drop_down .= '</select>';

            return $drop_down;

        }

        /**
         * @param $product_ids
         *
         * @return array|string
         */
        private function sanitize_product_ids($product_ids) {
            if (is_array($product_ids)) {
                $return = [];
            } else {
                return '';
            }

            foreach ($product_ids as $key => $product_id) {
                if (is_numeric($product_id)) {
                    $return[] = $product_id;
                }
            }

            return $return;
        }

        public function admin_notices() {
            $enable_debugging = (boolean) get_option(WC_PPP_SLUG.'_enable_debugging', false);

            if ($enable_debugging) {
                $class   = 'notice notice-error is-dismissible';
                $message = __('You currently have the WooCommerce Pay for Post Debug enabled.  This can pose a performance and security risk.  Please disable if not actively debugging.', 'sample-text-domain');
                printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), esc_html($message));
            }

        }

    }
