<?php /** @noinspection PhpIncludeInspection */

    /** @noinspection ALL */

    class Woocommerce_Pay_Per_Post {

	protected $loader;
	protected $plugin_name;
	protected $version;
	protected $template_path;

	public function __construct() {

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	public function run() {
		$this->loader->run();
	}

	public function get_loader() {
		return $this->loader;
	}

	private function define_admin_hooks() {

		$plugin_admin = new Woocommerce_Pay_Per_Post_Admin();

		$this->loader->add_action( 'admin_menu', $plugin_admin, 'add_plugin_options' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'options_init' );
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'admin_notices' );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'admin_head', $plugin_admin, 'admin_menu_separator_styles' );

		$this->loader->add_action( 'add_meta_boxes', $plugin_admin, 'meta_box' );
		$this->loader->add_action( 'save_post', $plugin_admin, 'save_meta_box' );
		$this->loader->add_action( 'edit_attachment', $plugin_admin, 'save_meta_box' );

		$this->loader->add_action( 'in_plugin_update_message-woocommerce-pay-per-post/woocommerce-pay-per-post.php', $plugin_admin, 'prefix_plugin_update_message', 10, 2 );

		foreach ( $plugin_admin->get_post_types() as $post_type ) {
			$this->loader->add_action( 'manage_' . $post_type . '_posts_custom_column', $plugin_admin, 'manage_custom_column', 10, 2 );
			$this->loader->add_filter( 'manage_' . $post_type . '_posts_columns', $plugin_admin, 'manage_columns' );
			$this->loader->add_filter( 'manage_edit-' . $post_type . '_sortable_columns', $plugin_admin, 'sortable_columns' );
		}

		$this->loader->add_filter( 'plugin_action_links_woocommerce-pay-per-post/woocommerce-pay-per-post.php', $plugin_admin, 'plugin_settings_link', 10, 2 );

		if ( wcppp_freemius()->is__premium_only() ) {
			if ( wcppp_freemius()->can_use_premium_code() ) {
				$this->loader->add_action( 'show_user_profile', $plugin_admin, 'extra_user_profile_fields__premium_only' );
				$this->loader->add_action( 'edit_user_profile', $plugin_admin, 'extra_user_profile_fields__premium_only' );
				$this->loader->add_action( 'wp_ajax_delete_page_view', $plugin_admin, 'ajax_delete_page_view__premium_only' );
				$this->loader->add_action( 'update_option_' . WC_PPP_SLUG . '_display_on_my_account', $plugin_admin, 'maybe_flush_permalinks__premium_only', 10, 3 );


			}
		}

	}

	private function define_public_hooks() {

		$plugin_public = new Woocommerce_Pay_Per_Post_Public();

        $this->loader->add_action( 'init', $plugin_public, 'init' );
		$this->loader->add_action( 'get_header', $plugin_public, 'should_disable_comments' );
		$this->loader->add_action( 'template_redirect', $plugin_public, 'set_product_ids' );

		$this->loader->add_filter( 'the_content', $plugin_public, 'restrict_content' );


		if ( wcppp_freemius()->is__premium_only() ) {
			if ( wcppp_freemius()->can_use_premium_code() ) {
				$integration_woocommerce = new \PRAMADILLO\INTEGRATIONS\WooCommerce();

				$this->loader->add_filter( 'woocommerce_display_item_meta', $integration_woocommerce, 'get_purchased_content_by_product_id', 10, 2 );

				$display_on_my_account = (bool) get_option( WC_PPP_SLUG . '_display_on_my_account', false );
				$show_locks            = (bool) get_option( WC_PPP_SLUG . '_show_locks', false );

				if ( $show_locks ) {
					$this->loader->add_filter( 'the_title', $plugin_public, 'add_locks_to_post_title', 10, 2 );
				}

				if ( $display_on_my_account ) {
					$this->loader->add_action( 'init', $integration_woocommerce, 'purchased_content_endpoint__premium_only' );
					$this->loader->add_filter( 'query_vars', $integration_woocommerce, 'purchased_content_query_vars__premium_only' );
					$this->loader->add_filter( 'woocommerce_account_menu_items', $integration_woocommerce, 'add_purchased_content_link_to_my_account__premium_only', 15 );
					$this->loader->add_action( 'woocommerce_account_purchased-content_endpoint', $integration_woocommerce, 'display_purchased_content_in_my_account__premium_only' );

				}

			}
		}

	}

	private function load_dependencies() {

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woocommerce-pay-per-post-loader.php';
        /** @noinspection PhpIncludeInspection */
        require_once plugin_dir_path(dirname(__FILE__ ) ).'includes/class-woocommerce-pay-per-post-i18n.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woocommerce-pay-per-post-helper.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woocommerce-pay-per-post-protection-checks.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woocommerce-pay-per-post-logger.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woocommerce-pay-per-post-deprecated.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-woocommerce-pay-per-post-admin.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-woocommerce-pay-per-post-debug.php';

		if ( wcppp_freemius()->is__premium_only() ) {
			if ( wcppp_freemius()->can_use_premium_code() ) {

				require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-woocommerce-pay-per-post-users-table__premium_only.php';

				/**
				 * Premium Integrations
				 */
				if ( Woocommerce_Pay_Per_Post_Helper::can_use_woocommerce_memberships() ) {
					require_once plugin_dir_path( dirname( __FILE__ ) ) . 'integrations/woocommerce-memberships/WooCommerceMemberships.php';
				}
				if ( Woocommerce_Pay_Per_Post_Helper::can_use_woocommerce_subscriptions() ) {
					require_once plugin_dir_path( dirname( __FILE__ ) ) . 'integrations/woocommerce-subscriptions/WooCommerceSubscriptions.php';
				}
				if ( Woocommerce_Pay_Per_Post_Helper::can_use_paid_membership_pro() ) {
					require_once plugin_dir_path( dirname( __FILE__ ) ) . 'integrations/paid-memberships-pro/PaidMembershipsPro.php';
				}

                if (Woocommerce_Pay_Per_Post_Helper::can_use_elementor()) {
                    require_once plugin_dir_path(dirname(__FILE__)).'integrations/elementor/Elementor.php';
                }
			}
		}

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'integrations/woocommerce/WooCommerce.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-woocommerce-pay-per-post-public.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-woocommerce-pay-per-post-restrict-content.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-woocommerce-pay-per-post-shortcodes.php';

		$this->loader = new Woocommerce_Pay_Per_Post_Loader();

	}

	private function set_locale() {

		$plugin_i18n = new Woocommerce_Pay_Per_Post_i18n();
		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

}
