<?php


namespace PRAMADILLO\INTEGRATIONS;


use Woocommerce_Pay_Per_Post_Helper;

class WooCommerce {

	private $available_templates;

	public function __construct() {
		$this->available_templates = array(
			'woocommerce-order-receipt' => 'woocommerce-order-receipt.php'
		);

		if ( wcppp_freemius()->is__premium_only() ) {
			if ( wcppp_freemius()->can_use_premium_code() ) {
				$this->available_templates['purchased-polylang']                 = 'shortcode-purchased-polylang.php';
				$this->available_templates['woocommerce-order-receipt-polylang'] = 'woocommerce-order-receipt-polylang.php';
			}
		}
	}

	public function purchased_content_endpoint__premium_only() {
		add_rewrite_endpoint( 'purchased-content', EP_ROOT | EP_PAGES );
	}

	public function purchased_content_query_vars__premium_only( $vars ) {
		$vars[] = 'purchased-content';

		return $vars;
	}

	public function display_purchased_content_in_my_account__premium_only() {
		do_action( 'wc_pay_per_post_before_my_account_shortcode' );
		if(apply_filters( 'wc_pay_per_post_my_account_show_has_access', true)) {
			echo '<h3>' . apply_filters( 'wc_pay_per_post_my_account_has_access_title', esc_html__( 'Content you have access to', 'wc_pay_per_post' ) ) . '</h3>';
			echo do_shortcode( ' [woocommerce-payperpost template="has_access"] ' );
		}
		if(apply_filters( 'wc_pay_per_post_my_account_show_purchased', true)) {
			echo '<h3>' . apply_filters( 'wc_pay_per_post_my_account_purchased_title', esc_html__( 'Content purchased', 'wc_pay_per_post' ) ) . '</h3>';
			echo do_shortcode( ' [woocommerce-payperpost template="purchased"] ' );
		}
		do_action( 'wc_pay_per_post_after_my_account_shortcode' );
	}

	/**
	 * @param $items
	 *
	 * @return mixed
	 */
	public function add_purchased_content_link_to_my_account__premium_only( $items ) {
		$items['purchased-content'] = apply_filters( 'wc_pay_per_post_my_account_tab_title', esc_html__( 'Purchased Content', 'wc_pay_per_post' ) );

		Woocommerce_Pay_Per_Post_Helper::move_to_top( $items, 'purchased-content' );
		Woocommerce_Pay_Per_Post_Helper::move_to_top( $items, 'dashboard' );

		return $items;
	}

	/**
	 * @param $formatted_meta
	 * @param $item
	 *
	 * @return string
	 */
	public function get_purchased_content_by_product_id( $formatted_meta, $item ): string {

		if ( $item->get_variation_id() > 0 ){
			$purchased = Woocommerce_Pay_Per_Post_Helper::get_posts_associated_with_product_id( $item->get_variation_id() );
		} else {
			$purchased = Woocommerce_Pay_Per_Post_Helper::get_posts_associated_with_product_id( $item->get_product_id() );
		}

		$template  = 'woocommerce-order-receipt';

		if ( ! empty( $purchased ) ) {

			if ( wcppp_freemius()->is__premium_only() ) {
				if ( wcppp_freemius()->can_use_premium_code() ) {

					if ( function_exists( 'pll_current_language' ) && get_option( WC_PPP_SLUG . '_enable_polylang', false ) ) {
						$template = 'woocommerce-order-receipt-polylang';
					}

				}
			}

			ob_start();

            /** @noinspection PhpIncludeInspection */
            require Woocommerce_Pay_Per_Post_Helper::locate_template($this->available_templates[$template ], '', WC_PPP_PATH.'public/partials/' );

			return ob_get_clean();
		}

		return $formatted_meta;

	}

	static public function hide_item_meta_from_email( $css ): string {
		return $css . '.ppp-purchased-content {display:none;}';
	}

}