<?php

    use PRAMADILLO\Woocommerce_Pay_Per_Post_Restrict_Content;

    /**
     * Class Woocommerce_Pay_Per_Post_Public
     */
    class Woocommerce_Pay_Per_Post_Public {

        private $should_track_pageview;

        public function init() {
            Woocommerce_Pay_Per_Post_Shortcodes::register_shortcodes();

            if (wcppp_freemius()->is__premium_only()) {
                if (wcppp_freemius()->can_use_premium_code()) {

                    if (apply_filters('wc_pay_per_post_hide_item_meta_in_email', false)) {
                        add_filter('woocommerce_email_styles', ['PRAMADILLO\INTEGRATIONS\WooCommerce', 'hide_item_meta_from_email']);
                    }

                }
            }

        }


        public function load_integrations() {
            $post = get_post();
            if (Woocommerce_Pay_Per_Post_Helper::can_use_elementor()) {
//                if (is_admin() || \Elementor\Plugin::$instance->documents->get($post->ID)->is_built_with_elementor()) {
                    require_once plugin_dir_path(dirname(__FILE__)).'integrations/elementor/Elementor.php';
//                }
            }
        }

        public function add_locks_to_post_title($title, $post_id) {

            if ( ! Woocommerce_Pay_Per_Post_Helper::has_access($post_id, false) && ! is_admin()) {
                if (in_the_loop()) {
                    return apply_filters('wc_pay_per_post_paywall_icon', '<span>&#x1F512;</span>').' '.$title;
                }
            }

            return $title;
        }


        public function should_disable_comments() {

            $turn_off_comments_completely_to_everyone_on_protected_posts = (bool) get_option(WC_PPP_SLUG.'_turn_off_comments_when_protected', true);
            $allow_admins_access_to_protected_posts                      = get_option(WC_PPP_SLUG.'_allow_admins_access_to_protected_posts', false);

            $is_protected = Woocommerce_Pay_Per_Post_Helper::is_protected(get_the_ID());

            if ($turn_off_comments_completely_to_everyone_on_protected_posts && $is_protected) {
                add_filter('comments_open', function () {
                    return false;
                });
                add_filter('get_comments_number', function () {
                    return 0;
                });
            }

            if (wcppp_freemius()->is__premium_only()) {
                if (wcppp_freemius()->can_use_premium_code()) {

                    if ($is_protected) {
                        $turn_off_comments_to_people_that_have_not_purchased_post = (bool) get_option(WC_PPP_SLUG.'_turn_off_comments_public', false);

                        if ($turn_off_comments_to_people_that_have_not_purchased_post) {
                            $this->should_track_pageview = false;

                            if (Woocommerce_Pay_Per_Post_Helper::has_access(null, false) || ($allow_admins_access_to_protected_posts && $this->is_admin())) {
                                add_filter('comments_open', function () {
                                    return true;
                                });
                            } else {
                                add_filter('comments_open', function () {
                                    return false;
                                });
                            }

                        }

                    }

                }
            }
        }

        /**
         * @param $unfiltered_content
         *
         * @return string
         */
        public function restrict_content($unfiltered_content): string {


            //ensure that our filter only runs one time
            if ( ! in_the_loop() && ! is_singular() && ! is_main_query() && in_array(get_post_type(), Woocommerce_Pay_Per_Post_Helper::get_post_types())) {
                return $unfiltered_content;
            }

            Woocommerce_Pay_Per_Post_Helper::logger('the_content() called, reached restriction_content function');

            if ( ! Woocommerce_Pay_Per_Post_Helper::is_protected()) {
                Woocommerce_Pay_Per_Post_Helper::logger('skipping - Is not protected.');

                return $unfiltered_content;
            }
            //check and see if inline shortcode exists, if it does skip
            if (strpos($unfiltered_content, '[/wc-pay-for-post-inline]')) {
                Woocommerce_Pay_Per_Post_Helper::logger('skipping - Inline Shortcode Present');

                // Handle shortcode access
                return $unfiltered_content;
            } elseif (strpos($unfiltered_content, '[products ids=')) {
                Woocommerce_Pay_Per_Post_Helper::logger('skipping - Shortcode [product present');

                //check and see if the products shortcode exists, if it does skip.  This is to account for Elementor Elements being protected
                return $unfiltered_content;
            } else {
                $restrict = new Woocommerce_Pay_Per_Post_Restrict_Content();

                $show_paywall = apply_filters('wc_pay_per_post_force_bypass_paywall', $restrict->can_user_view_content());

                if ($show_paywall == false) {
                    return $restrict->show_content($unfiltered_content);
                }
                remove_filter(current_filter(), __FUNCTION__);

                return $restrict->show_paywall($unfiltered_content);
            }

        }

        public function set_product_ids() {
            global $product_ids;
            $product_ids = (array) get_post_meta(get_the_ID(), WC_PPP_SLUG.'_product_ids', true);
            //Elementor Integration
            if (wcppp_freemius()->is__premium_only()) {
                if (wcppp_freemius()->can_use_premium_code()) {
                    if (Woocommerce_Pay_Per_Post_Helper::can_use_elementor()) {
                        $elementor_data = json_decode( (string) get_post_meta(get_the_ID(), '_elementor_data', true));
                        if ( ! is_null($elementor_data)) {
                            $elementor_product_ids = Woocommerce_Pay_Per_Post_Helper::recursive_array_search('wc_pay_per_post_select_products', $elementor_data);
                            if ($elementor_product_ids && count($elementor_product_ids) > 0) {
                                $product_ids = array_merge($product_ids, $elementor_product_ids);
                            }
                        }
                    }
                }
            }
        }

        protected function show_comments($has_purchased = false) {
            $turn_off_comments_when_protected       = get_option(WC_PPP_SLUG.'_turn_off_comments_when_protected', true);
            $allow_admins_access_to_protected_posts = get_option(WC_PPP_SLUG.'_allow_admins_access_to_protected_posts', false);

            if ($turn_off_comments_when_protected) {
                add_filter('comments_open', [$this, 'comments_closed']);
            }

            if (wcppp_freemius()->is__premium_only()) {
                if (wcppp_freemius()->can_use_premium_code()) {
                    $turn_off_comments_public = get_option(WC_PPP_SLUG.'_turn_off_comments_public', false);

                    if (($turn_off_comments_public && $has_purchased) || ($turn_off_comments_public && $allow_admins_access_to_protected_posts && $this->is_admin())) {
                        add_filter('comments_open', [$this, 'comments_open']);
                    } else {
                        add_filter('comments_open', [$this, 'comments_closed']);

                    }
                }
            }

        }

        /**
         * @return bool
         */
        protected function is_admin(): bool {
            $current_user = wp_get_current_user();
            if (user_can($current_user, 'administrator')) {
                return true;
            }

            return false;
        }

    }
