<?php

use Elementor\Plugin;
use PRAMADILLO\Woocommerce_Pay_Per_Post_Restrict_Content;


/**
 * Class Woocommerce_Pay_Per_Post_Shortcodes
 */
class Woocommerce_Pay_Per_Post_Shortcodes {

	static public function register_shortcodes() {
		add_shortcode( 'woocommerce-payperpost', [ __CLASS__, 'process_shortcode' ] );
		add_shortcode( 'wc-pay-for-post', [ __CLASS__, 'process_shortcode' ] );

		if ( wcppp_freemius()->is__premium_only() ) {
			if ( wcppp_freemius()->can_use_premium_code() ) {
				add_shortcode( 'wc-pay-for-post-inline', [ __CLASS__, 'process_inline_shortcode__premium_only' ] );
				add_shortcode( 'wc-pay-for-post-product-unlocks', [
					__CLASS__,
					'process_product_unlocks_shortcode__premium_only',
				] );
			}
		}
	}

	static public function process_product_unlocks_shortcode__premium_only() {
		global $product;
		$posts = [];

		if ( is_null( $product ) ) { //This is to account for when accessing the product via /wp-json/wc/v2/products/
			return false;
		}

		if ( $product->is_type( 'variable' ) ) {
			$variations     = $product->get_available_variations();
			$variations_ids = wp_list_pluck( $variations, 'variation_id' );

			foreach ( $variations_ids as $variationsId ) {
				$posts = array_merge( Woocommerce_Pay_Per_Post_Helper::get_posts_associated_with_product_id( $product->get_id() ), Woocommerce_Pay_Per_Post_Helper::get_posts_associated_with_product_id( $variationsId ) );
			}
		} else {
			$id    = $product->get_id();
			$posts = Woocommerce_Pay_Per_Post_Helper::get_posts_associated_with_product_id( $id );
		}

		$return = null;

		if ( count( $posts ) > 0 ) {
			$return = '<ul class="wc-pay-for-post-unlocks-posts">';
			foreach ( $posts as $post ) {
				$status = 'locked';
				if ( Woocommerce_Pay_Per_Post_Helper::has_access( $post['post_id'], false ) ) {
					$status = 'unlocked';
				}
				$return .= '<li class="' . $status . '"><a href="' . $post['post_url'] . '">' . $post['post_title'] . '</a></li>';
			}
			$return .= '</ul>';
		}


		return $return;
	}

	static public function process_inline_shortcode__premium_only( $atts, $content = "" ) {
		//Check to see if Elementor is loaded
		if ( did_action( 'elementor/loaded' ) ) {
			if ( Plugin::$instance->editor->is_edit_mode() || Plugin::$instance->preview->is_preview_mode() ) {
				return false;
			}
		}

		$restrict     = new Woocommerce_Pay_Per_Post_Restrict_Content();
		$show_paywall = apply_filters( 'wc_pay_per_post_force_bypass_paywall', $restrict->can_user_view_content() );
		//TODO Add in a way to by pass the paywall content on the frontend, but all for it to be used via PHP.  Possibly a checkbox on the metabox that says disable protection checking?
		if ( isset( $atts['paywall_message'] ) ) {
			global $paywall_message;
			$paywall_message = $atts['paywall_message'];
			add_filter( 'wc_pay_for_post_override_paywall_content', [
				__CLASS__,
				'handle_inline_paywall_message__premium_only',
			], 10, 1 );
		} else {
			remove_filter( 'wc_pay_for_post_override_paywall_content', [
				__CLASS__,
				'handle_inline_paywall_message__premium_only',
			] );
		}
		if ( $show_paywall == false ) {
			if ( apply_filters( 'wc_pay_per_post_apply_the_content_filter_to_inline_shortcode', true ) ) {
				return apply_filters( 'the_content', $restrict->show_content( $content ) );
			} else {
				return $restrict->show_content( $content );
			}

		}

		return $restrict->show_paywall( $content );
	}

	static public function handle_inline_paywall_message__premium_only( $paywall_message ) {
		global $paywall_message, $product_ids;
		$add_to_cart = [];
		foreach ( (array) $product_ids as $product_id ) {

			$add_to_cart[] = do_shortcode( '[add_to_cart id="' . $product_id . '"]' );
		}

		$add_to_cart = implode( ', ', $add_to_cart );

		$links = [];
		foreach ( (array) $product_ids as $product_id ) {
			$product = wc_get_product( $product_id );
			if ( $product ) {
				$link_name = apply_filters( 'wc_pay_per_post_inline_shortcode_product_link_name', 'Add ' . $product->get_name() . ' to Cart', $product->get_name() );
				$links[]   = '<a href="/' . do_shortcode( '[add_to_cart_url id="' . $product_id . '"]' ) . '">' . $link_name . '</a>';
			}
		}

		$links           = implode( ', ', $links );
		$products        = do_shortcode( '[products ids="' . implode( ',', (array) $product_ids ) . '"]' );
		$paywall_message = str_replace( '{{products}}', $products, $paywall_message );
		$paywall_message = str_replace( '{{add_to_cart}}', $add_to_cart, $paywall_message );
		$paywall_message = str_replace( '{{add_to_cart_link}}', $links, $paywall_message );

		return str_replace( '{{add_to_cart_link}}', $links, $paywall_message );
	}

	static public function process_shortcode( $atts ) {
		global $product_ids;
		$template = 'purchased';
		$orderby  = 'post_date';
		$order    = 'DESC';

		if ( wcppp_freemius()->is__premium_only() ) {
			if ( wcppp_freemius()->can_use_premium_code() ) {

				if ( isset( $atts['order'] ) && ( $atts['order'] === 'ASC' || $atts['order'] === 'DESC' ) ) {
					$order = $atts['order'];
				}

				if ( isset( $atts['orderby'] ) && ( $atts['orderby'] === 'ID' || $atts['orderby'] === 'title' || $atts['orderby'] === 'post_date' || $atts['orderby'] === 'menu_order' ) ) {
					$orderby = $atts['orderby'];
				}

				if ( isset( $atts['product_ids'] ) ) {
					$product_ids = explode( ',', $atts['product_ids'] );
				}

			}
		}

		if ( isset( $atts['template'] ) && array_key_exists( $atts['template'], self::available_templates() ) ) {
			$template = $atts['template'];
		}

		$custom_post_types = get_option( WC_PPP_SLUG . '_custom_post_types', [] );
		$custom_post_types = empty( $custom_post_types ) ? [] : $custom_post_types;

		if ( ! is_array( $custom_post_types ) ) {
			$custom_post_types = explode( ',', $custom_post_types );
		}

		$args = [
			'orderby'     => $orderby,
			'order'       => $order,
			'nopaging'    => true,
			'meta_query'  => [
				[
					'key'     => WC_PPP_SLUG . '_product_ids',
					'value'   => '',
					'compare' => '!=',
				],
			],
			'post_status' => 'publish',
			'post_type'   => $custom_post_types,
		];


		if ( wcppp_freemius()->is__premium_only() ) {
			if ( wcppp_freemius()->can_use_premium_code() ) {

				$args['meta_query']['relation'] = 'OR';

				//Account for Elementor
				$args['meta_query'][] = [
					'key'     => '_elementor_data',
					'value'   => '"wc_pay_per_post_enable":"yes"', //TODO There has to be a better way to do this.
					'compare' => 'LIKE',
				];

				if ( isset( $atts['category_slug'] ) ) {
					$args['category_name'] = $atts['category_slug'];
				}

				if ( isset( $atts['tag'] ) ) {
					$args['tag'] = $atts['tag'];
				}

				if ( Woocommerce_Pay_Per_Post_Helper::can_use_polylang__premium_only() ) {
					$args['lang'] = '';
				}

				if ( isset( $atts['product_ids'] ) ) {
					unset( $args['meta_query'] );

					$args['meta_query'] = [
						'relation' => 'OR',
					];

					foreach ( $product_ids as $product_id ) {
						$args['meta_query'][] = [
							'key'     => WC_PPP_SLUG . '_product_ids',
							'value'   => sprintf( '^%1$s$|s:%2$u:"%1$s";', $product_id, strlen( $product_id ) ),
							'compare' => 'REGEXP',
						];
						$args['meta_query'][] = [
							'key'     => '_elementor_data',
							'value'   => '"' . $product_id . '"',
							'compare' => 'LIKE',
						];
					}

				}

			}
		}
		Woocommerce_Pay_Per_Post_Helper::logger( 'process_shortcode() args' );
		Woocommerce_Pay_Per_Post_Helper::logger( var_export( $args, true ) );
		$get_ppp_args = apply_filters( 'wc_pay_per_post_args', $args );

		Woocommerce_Pay_Per_Post_Helper::logger( 'process_shortcode() args AFTER FILTER - wc_pay_per_post_args' );
		Woocommerce_Pay_Per_Post_Helper::logger( var_export( $args, true ) );

		if ( wcppp_freemius()->is__premium_only() ) {
			if ( wcppp_freemius()->can_use_premium_code() ) {

				if ( isset( $atts['category_slug'] ) ) {
					$transient = $atts['category_slug'];
				}

				if ( isset( $atts['tag'] ) ) {
					$transient = $atts['tag'];
				}

			}
		}

		$ppp_posts = Woocommerce_Pay_Per_Post_Helper::get_protected_posts( $args, $transient );

		ob_start();
		switch ( $template ) {
			case 'has_access':
				self::shortcode_has_access( $template, $ppp_posts );
				break;
			case 'purchased':
				self::shortcode_purchased( $template, $ppp_posts );
				break;
			case 'remaining':
				self::shortcode_remaining( $template, $ppp_posts );
				break;
			case 'all':
				self::shortcode_all( $template, $ppp_posts );
				break;
		}

		return ob_get_clean();

	}

	/**
	 * @param $template
	 * @param $ppp_posts
	 */
	protected static function shortcode_purchased( $template, $ppp_posts ) {
		$purchased = [];
		if ( is_user_logged_in() ) {
			foreach ( $ppp_posts as $post ) {
				if ( Woocommerce_Pay_Per_Post_Helper::has_purchased( $post->ID, false ) ) {
					if ( wcppp_freemius()->is__premium_only() ) {
						if ( wcppp_freemius()->can_use_premium_code() ) {
							$restrict                    = new Woocommerce_Pay_Per_Post_Restrict_Content();
							$post->{"expiration_status"} = $restrict->is_expired( $post->ID );
						}
					}
					$purchased[] = $post;
				}
			}

			if ( wcppp_freemius()->is__premium_only() ) {
				if ( wcppp_freemius()->can_use_premium_code() ) {

					if ( function_exists( 'pll_current_language' ) && get_option( WC_PPP_SLUG . '_enable_polylang', false ) ) {
						$template = 'purchased-polylang';
					}

				}
			}

			/** @noinspection PhpIncludeInspection */
			require Woocommerce_Pay_Per_Post_Helper::locate_template( self::available_templates()[ $template ], '', WC_PPP_PATH . 'public/partials/' );
		}

	}

	/**
	 * @param $template
	 * @param $ppp_posts
	 */
	protected static function shortcode_has_access( $template, $ppp_posts ) {
		$purchased = [];
		if ( is_user_logged_in() ) {
			foreach ( $ppp_posts as $post ) {
				Woocommerce_Pay_Per_Post_Helper::logger( 'Woocommerce_Pay_Per_Post_Helper::has_access($post->ID - ' . print_r( $post->ID, true ) );
				if ( Woocommerce_Pay_Per_Post_Helper::has_access( $post->ID, false ) ) {
					$purchased[] = $post;
				}

			}

			if ( wcppp_freemius()->is__premium_only() ) {
				if ( wcppp_freemius()->can_use_premium_code() ) {

					if ( function_exists( 'pll_current_language' ) && get_option( WC_PPP_SLUG . '_enable_polylang', false ) ) {
						$template = 'has_access-polylang';
					}

				}
			}

			/** @noinspection PhpIncludeInspection */
			require Woocommerce_Pay_Per_Post_Helper::locate_template( self::available_templates()[ $template ], '', WC_PPP_PATH . 'public/partials/' );
		}

	}

	/**
	 * @param $template
	 * @param $ppp_posts
	 */
	protected static function shortcode_remaining( $template, $ppp_posts ) {

		$remaining = [];

		if ( is_user_logged_in() ) {

			foreach ( $ppp_posts as $post ) {
				if ( ! Woocommerce_Pay_Per_Post_Helper::has_access( $post->ID, false ) ) {
					$remaining[] = $post;
				}
			}

			/** @noinspection PhpIncludeInspection */
			require Woocommerce_Pay_Per_Post_Helper::locate_template( self::available_templates()[ $template ], '', WC_PPP_PATH . 'public/partials/' );

		}

	}

	/**
	 * @param $template
	 * @param $ppp_posts
	 */
	protected static function shortcode_all( $template, $ppp_posts ) {

		/** @noinspection PhpIncludeInspection */
		require Woocommerce_Pay_Per_Post_Helper::locate_template( self::available_templates()[ $template ], '', WC_PPP_PATH . 'public/partials/' );

	}

	/**
	 * @param null $post_id
	 *
	 * @return bool
	 */
	protected static function has_purchased_products( $post_id = null ): bool {
		_deprecated_function( __FUNCTION__, '2.6.2', 'Woocommerce_Pay_Per_Post_Helper::has_purchased()' );
		global $product_ids;
		if ( is_null( $post_id ) ) {
			$post_id = get_the_ID();
		}

		$current_user = wp_get_current_user();
		foreach ( (array) $product_ids as $id ) {

			Woocommerce_Pay_Per_Post_Helper::logger( '_deprecated_function Looking to see if purchased product ' . trim( $id ) );

			if ( wc_customer_bought_product( $current_user->user_email, $current_user->ID, trim( $id ) ) ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * @param $post_id
	 *
	 * This returns the INVERSE of can_user_view_content()
	 *
	 * @return bool
	 */
	protected static function has_access( $post_id ): bool {
		$restrict = new Woocommerce_Pay_Per_Post_Restrict_Content( $post_id );
		$restrict->set_track_pageview( false );

		if ( apply_filters( 'wc_pay_per_post_hide_delay_restricted_posts_when_paywall_should_not_be_shown', false ) ) {
			/**
			 * We have the following check because if you have delay protection enabled and the post is not suppose to show the paywall for
			 * a year after publishing, the posts that show in the purchased content tab or purchased shortcode will output all of the ppp posts
			 * that have delay protection even though they are not suppose to show paywall yet.
			 */
			if ( 'delay' === Woocommerce_Pay_Per_Post_Helper::is_protected( $post_id ) ) {
				return $restrict->check_if_should_show_paywall();
			}
		}

		return ! $restrict->can_user_view_content();
	}

	private static function available_templates(): array {
		return [
			'purchased'  => 'shortcode-purchased.php',
			'has_access' => 'shortcode-has_access.php',
			'all'        => 'shortcode-all.php',
			'remaining'  => 'shortcode-remaining.php',
		];
	}

}