<?php

/**
 * Allows for the sale of a specific post/page in WordPress through WooCommerce.
 *
 * @link                    pramadillo.com
 * @since                   2.0.0
 * @package                 Woocommerce_pay_per_post
 * @wordpress-plugin
 * Plugin Name: Pay For Post with WooCommerce Premium
 * Plugin URI:              pramadillo.com/plugins/woocommerce-pay-per-post
 * Description:             Allows for the sale of a specific post/page in WordPress through WooCommerce.
 * Version:                 3.0.6
 * WC requires at least:    2.6
 * WC tested up to:         5.5.2
 * Author:                  Pramadillo
 * Author URI:              pramadillo.com
 * License:                 GPL-2.0+
 * License URI:             http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:             wc_pay_per_post
 * Domain Path:             /languages
 */
if ( !defined( 'ABSPATH' ) ) {
    exit;
}
const  WC_PPP_VERSION = '3.0.6' ;
const  WC_PPP_SLUG = 'wc_pay_per_post' ;
const  WC_PPP_NAME = 'Pay For Post with WooCommerce' ;
const  WC_PPP_TEMPLATE_PATH = 'woocommerce-pay-per-post/' ;
define( 'WC_PPP_PATH', plugin_dir_path( __FILE__ ) );
define( 'WC_PPP_URL', plugin_dir_url( __FILE__ ) );
define( 'WC_PPP_BASENAME', plugin_basename( __FILE__ ) );

class wcpppFsNull {
    public function is__premium_only() {
        return true;
    }
    public function can_use_premium_code() {
        return true;
    }
    public function is_trial() {
        return false;
    }
    public function is_not_paying() {
        return false;
    }
    public function get_upgrade_url() {
        return '';
    }
}

if ( function_exists( 'wcppp_freemius' ) ) {
    wcppp_freemius()->set_basename( true, __FILE__ );
} else {
    if ( !function_exists( 'wcppp_freemius' ) ) {
        function wcppp_freemius()
        {
            global  $wcppp_freemius ;
            
            if ( !isset( $wcppp_freemius ) ) {
                // Activate multisite network integration.
                if ( !defined( 'WP_FS__PRODUCT_1664_MULTISITE' ) ) {
                    define( 'WP_FS__PRODUCT_1664_MULTISITE', true );
                }
                // Include Freemius SDK.
                /** @noinspection PhpIncludeInspection */
                require_once plugin_dir_path( __FILE__ ) . 'vendor/freemius/wordpress-sdk/start.php';
                try {
                    $wcppp_freemius = new wcpppFsNull();
                } catch ( Freemius_Exception $e ) {
                    die( esc_html( $e->getMessage() ) );
                }
            }
            
            return $wcppp_freemius;
        }
    
    }
    /** @noinspection PhpIncludeInspection */
    require plugin_dir_path( __FILE__ ) . 'includes/class-woocommerce-pay-per-post.php';
    /** @noinspection PhpIncludeInspection */
    require plugin_dir_path( __FILE__ ) . 'vendor/autoload.php';
    function activate_woocommerce_pay_per_post()
    {
        /** @noinspection PhpIncludeInspection */
        require_once plugin_dir_path( __FILE__ ) . 'includes/class-woocommerce-pay-per-post-activator.php';
        Woocommerce_Pay_Per_Post_Activator::activate( WC_PPP_VERSION );
    }
    
    function deactivate_woocommerce_pay_per_post()
    {
        /** @noinspection PhpIncludeInspection */
        require_once plugin_dir_path( __FILE__ ) . 'includes/class-woocommerce-pay-per-post-deactivator.php';
        Woocommerce_Pay_Per_Post_Deactivator::deactivate();
    }
    
    register_activation_hook( __FILE__, 'activate_woocommerce_pay_per_post' );
    register_deactivation_hook( __FILE__, 'deactivate_woocommerce_pay_per_post' );
    function run_woocommerce_pay_per_post()
    {
        wcppp_freemius();
        do_action( 'wcppp_freemius_loaded' );
        add_filter( 'plugin_icon', 'wcppp_fs_custom_icon' );
        $plugin = new Woocommerce_Pay_Per_Post();
        $plugin->run();
    }
    
    function wcppp_fs_custom_icon() : string
    {
        return dirname( __FILE__ ) . '/admin/img/icon.png';
    }
    
    run_woocommerce_pay_per_post();
}
